unit Dbutil;  { Freeware (c) by Martin Austermeier .\\.\ 05/95 - no warranties }

{ component TLocator:
    Searches a TDataSet (=TTable or TQuery) sequentially
    for arbitrary data (because you specify an
    OnCheckData procedure; see below)

    Usage:
    + you might want to rename the component page
      in the procedure "Register" below, because it
      specified the german name of a palette page.

    + install this unit using Options/Install Component

    + insert a TLocator into a form

    + set its DATASET property

    + set SearchFromStart=FALSE if you dont wish
      to search from the beginning of the table

    + define an OnCheckData event, where you
      set "found" to TRUE, if the current record
      contains the searched data. This procedure
      is called for each record in your table, until
      it returns "found" or EoF.

      Example for an OnCheckData procedure:

      procedure TForm1.Locator1CheckData(var found : Boolean);
      begin
        if (table1custNo.value = 123) then
          found := TRUE;
      end;

      Or, define a private variable in your form like
        searchedCustNo : Word;
      And in your OnCheckData:
        found := (table1custNo.value = searchedCustNo);

      Of course, you may use any type of comparison,
      like "Pos('abc', table1Name.value) <> 0"..

    + call Locator.FindData in your program. Ex.:

        searchedCustNo := 123; (see ex. above)
        customerFound := locator1.FindData;
        (or: if locator1.FindData then ..)
}

interface
uses
  Classes, DB;

type
  TCheckDataEvent = procedure(var found : Boolean) of object;

type
  TLocator = class(TComponent)
  private
    fDataSet : TDataSet;
    fStartOpt : Boolean;
    fSuccess : Boolean;
    fOnCheckData : TCheckDataEvent;

    function GetTable : TDataSet;
    procedure SetTable(aTable : TDataSet);
    procedure SetStartOpt(enable : Boolean);
    procedure SetCheckDataEvent(aOnCheckData : TCheckDataEvent);
    procedure SetSuccess(aSuccess : Boolean);

  published
    property DataSet : TDataSet read GetTable write SetTable;
    property SearchFromStart : Boolean read fStartOpt write SetStartOpt;

    property OnCheckData : TCheckDataEvent read fOnCheckData write SetCheckDataEvent;

   public
     property Success : Boolean read fSuccess write SetSuccess;

     constructor Create(aOwner : TComponent); override;
     function FindData : Boolean; virtual;

  end;

{---------------------------}
procedure Register;
{---------------------------}

implementation


constructor TLocator.Create(aOwner : TComponent);
begin
  inherited Create(aOwner);
  fStartOpt := TRUE;
end;


function TLocator.GetTable : TDataSet;
begin
  result := fDataSet;
end;

procedure TLocator.SetTable(aTable : TDataSet);
begin
  fDataSet := aTable;
end;

procedure TLocator.SetStartOpt(enable : Boolean);
begin
  fStartOpt := enable;
end;


procedure TLocator.SetCheckDataEvent(aOnCheckData : TCheckDataEvent);
begin
  fOnCheckData := aOnCheckData;
end;


procedure TLocator.SetSuccess(aSuccess : Boolean);
begin
  fSuccess := aSuccess;
end;


function TLocator.FindData : Boolean;
var
  bm : TBookMark;
  fnd : Boolean;
begin
  result := FALSE;

  if (Assigned(fDataSet)
  and fDataSet.active
  and Assigned(fOnCheckData))  { otherwise no use searching }
  then begin

    bm := fDataSet.GetBookMark;
    fDataSet.DisableControls;
    if (fStartOpt) then  { if SearchFromStart }
      fDataSet.First;      { then rewind table }

    WHILE NOT (fDataSet.EoF) DO BEGIN
      fOnCheckData(result);  { call user proc }

      IF (result) THEN
        Break;

      fDataSet.Next;
    END;

    IF (NOT result) THEN
      fDataSet.GotoBookMark(bm);

    fDataSet.EnableControls;
    fDataSet.FreeBookMark(bm);
  end;
  success := result;
end;

procedure Register;
begin
  RegisterComponents('Datenzugriff', [TLocator]);
  { "Datenzugriff" = Data Access page }
end;


end.
